<?php

declare(strict_types=1);

namespace Drupal\user_selected_formatter\Model;

use Drupal\Core\Entity\EntityDisplayBase;
use Drupal\Core\Field\FieldDefinitionInterface;

/**
 * Data model for a user selected formatter preset.
 */
readonly class Preset {

  /**
   * Label display mode.
   *
   * @see \Drupal\field\FieldLabelOptionsTrait::getFieldLabelOptions()
   */
  private const LABEL_DISPLAY_OPTIONS = [
    'above',
    'inline',
    'hidden',
    'visually_hidden',
  ];

  private const DEFAULT_LABEL_DISPLAY = 'above';

  public function __construct(
    public string $id,
    public string $label,
    public string $formatterType,
    public string $formatterLabelDisplayMode = self::DEFAULT_LABEL_DISPLAY,
    public array $formatterSettings = [],
    public array $formatterThirdPartySettings = [],
  ) {
    if (!$id) {
      throw new \InvalidArgumentException('The "id" parameter is required.');
    }
    if (!$label) {
      throw new \InvalidArgumentException('The "label" parameter is required.');
    }
    if (!$formatterType) {
      throw new \InvalidArgumentException('The "formatterType" parameter is required.');
    }
    if (!in_array($formatterLabelDisplayMode, self::LABEL_DISPLAY_OPTIONS, TRUE)) {
      throw new \InvalidArgumentException('The "formatterLabelMode" parameter should be one of: "' . implode('", "', self::LABEL_DISPLAY_OPTIONS) . '".');
    }
  }

  /**
   * Creates a new preset value object from an array.
   *
   * @param array $data
   *   Data as an array.
   *
   * @return $this
   */
  public static function createFromArray(array $data): static {
    return new static(
      id: $data['id'],
      label: $data['label'],
      formatterType: $data['formatter']['type'],
      formatterLabelDisplayMode: $data['formatter']['label'] ?? self::DEFAULT_LABEL_DISPLAY,
      formatterSettings: $data['formatter']['settings'] ?? [],
      formatterThirdPartySettings: $data['formatter']['third_party_settings'] ?? [],
    );
  }

  /**
   * Converts the preset value object into an array.
   *
   * @return array
   *   The array representation of the preset value object.
   */
  public function toArray(): array {
    return [
      'id' => $this->id,
      'label' => $this->label,
      'formatter' => [
        'type' => $this->formatterType,
        'label' => $this->formatterLabelDisplayMode,
        'settings' => $this->formatterSettings,
        'third_party_settings' => $this->formatterThirdPartySettings,
      ],
    ];
  }

  /**
   * Generates options suitable for the formatter plugin instantiation.
   *
   * @param \Drupal\Core\Field\FieldDefinitionInterface $definition
   *   The field definition.
   * @param string $viewMode
   *   The view mode.
   *
   * @return array
   *   Options suitable for the formatter plugin instantiation.
   */
  public function getFormatterOptions(FieldDefinitionInterface $definition, string $viewMode = EntityDisplayBase::CUSTOM_MODE): array {
    return [
      'field_definition' => $definition,
      'configuration' => [
        'type' => $this->formatterType,
        'label' => $this->formatterLabelDisplayMode,
        'settings' => $this->formatterSettings,
        'third_party_settings' => $this->formatterThirdPartySettings,
        'weight' => 0,
      ],
      'view_mode' => $viewMode,
    ];
  }

  /**
   * Returns an array of defaults.
   *
   * @return array
   *   Defaults for value object.
   */
  public static function getDefaults(): array {
    return [
      'id' => NULL,
      'label' => NULL,
      'formatter' => [
        'type' => NULL,
        'label' => self::DEFAULT_LABEL_DISPLAY,
        'settings' => [],
        'third_party_settings' => [],
      ],
    ];
  }

}
