<?php

declare(strict_types=1);

namespace Drupal\user_selected_formatter\Plugin\Field\FieldFormatter;

use Drupal\Core\Config\Entity\ThirdPartySettingsInterface;
use Drupal\Core\Field\Attribute\FieldFormatter;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Field\FormatterInterface;
use Drupal\Core\Field\FormatterPluginManager;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\user_selected_formatter\Model\Preset;
use Drupal\user_selected_formatter\UserSelectedFormatterServiceInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the 'user_selected_formatter_default' formatter.
 *
 * @see \user_selected_formatter_field_formatter_info_alter()
 */
#[FieldFormatter(
  id: 'user_selected_formatter_default',
  label: new TranslatableMarkup('User selected formatter'),
  field_types: [],
)]
class UserSelectedFormatterDefault extends FormatterBase implements ContainerFactoryPluginInterface {

  /**
   * Static cache for preset entity.
   */
  protected Preset|false $preset;

  /**
   * Static cache for the underlying user configured formatter.
   */
  protected FormatterInterface|false $userSelectedFormatter;

  public function __construct(
    $plugin_id,
    $plugin_definition,
    FieldDefinitionInterface $field_definition,
    array $settings,
    $label,
    $view_mode,
    array $third_party_settings,
    protected FormatterPluginManager $formatterManager,
    protected UserSelectedFormatterServiceInterface $service,
  ) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $label, $view_mode, $third_party_settings);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): self {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['label'],
      $configuration['view_mode'],
      $configuration['third_party_settings'],
      $container->get('plugin.manager.field.formatter'),
      $container->get('user_selected_formatter'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function prepareView(array $entities_items): void {
    if ($formatter = $this->getUserSelectedFormatter($entities_items[0])) {
      $formatter->prepareView($entities_items);
      return;
    }
    parent::prepareView($entities_items);
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode): array {
    if ($formatter = $this->getUserSelectedFormatter($items)) {
      return $formatter->viewElements($items, $langcode);
    }
    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function view(FieldItemListInterface $items, $langcode = NULL): array {
    $elements = parent::view($items, $langcode);
    if ($preset = $this->getUserSelectedPreset($items)) {
      // Make sure we're using the 3rd-party settings stored in preset.
      $elements['#third_party_settings'] = $preset->formatterThirdPartySettings;
    }
    else {
      $elements['#third_party_settings'] = [];
    }
    return $elements;
  }

  /**
   * Returns the plugin instance of the user selected formatter.
   *
   * @param \Drupal\Core\Field\FieldItemListInterface $items
   *   The field values to be rendered.
   *
   * @return \Drupal\Core\Field\FormatterInterface|false
   *   A formatter plugin instance or FALSE.
   */
  protected function getUserSelectedFormatter(FieldItemListInterface $items): FormatterInterface|false {
    if (!isset($this->userSelectedFormatter)) {
      $this->userSelectedFormatter = FALSE;
      if ($preset = $this->getUserSelectedPreset($items)) {
        $options = $preset->getFormatterOptions($this->fieldDefinition, $this->viewMode);
        $this->userSelectedFormatter = $this->formatterManager->getInstance($options);
      }
    }
    return $this->userSelectedFormatter;
  }

  /**
   * Returns the preset used for this field.
   *
   * @param \Drupal\Core\Field\FieldItemListInterface $items
   *   The field values to be rendered.
   *
   * @return \Drupal\user_selected_formatter\Model\Preset|null
   *   The preset object, if one has been selected.
   */
  protected function getUserSelectedPreset(FieldItemListInterface $items): Preset|false {
    if (!isset($this->preset)) {
      \assert($this->fieldDefinition instanceof ThirdPartySettingsInterface);
      $this->preset = FALSE;
      if (!$items->isEmpty()) {
        $this->preset = $this->service->getPresetByTargetField($items->getEntity(), $this->fieldDefinition) ?: FALSE;
      }
    }
    return $this->preset;
  }

}
