<?php

declare(strict_types=1);

namespace Drupal\user_selected_formatter\Plugin\Field\FieldType;

use Drupal\Component\Plugin\ConfigurableInterface;
use Drupal\Core\Config\Entity\ConfigEntityStorage;
use Drupal\Core\Config\Entity\ThirdPartySettingsInterface;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityReferenceSelection\SelectionInterface;
use Drupal\Core\Field\Attribute\FieldType;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\Plugin\Field\FieldType\StringItem;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\field\Entity\FieldConfig;

/**
 * Field item storing a reference to a user selected formatter preset.
 */
#[FieldType(
  id: 'user_selected_formatter',
  label: new TranslatableMarkup('User Selected Formatter Preset'),
  description: new TranslatableMarkup('Reference to a formatter preset used to format a field'),
  category: 'general',
  default_formatter: 'user_selected_formatter_default',
  cardinality: 1,
)]
class UserSelectedFormatterItem extends StringItem {

  /**
   * {@inheritdoc}
   */
  public static function defaultStorageSettings(): array {
    return [
      'max_length' => ConfigEntityStorage::MAX_ID_LENGTH,
      'is_ascii' => TRUE,
    ] + parent::defaultStorageSettings();
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultFieldSettings(): array {
    return ['field' => NULL] + parent::defaultFieldSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function fieldSettingsForm(array $form, FormStateInterface $form_state): array {
    $handler = \Drupal::getContainer()->get('plugin.manager.entity_reference_selection')->createInstance('default:user_selected_formatter');
    \assert($handler instanceof SelectionInterface && $handler instanceof ConfigurableInterface);

    $fieldDefinition = $this->getFieldDefinition();
    $definitions = $this->getFieldManager()->getFieldDefinitions($fieldDefinition->getTargetEntityTypeId(), $fieldDefinition->getTargetBundle());

    foreach ($definitions as $fieldName => $definition) {
      if ($definition->getType() === 'user_selected_formatter') {
        $targetFieldName = $definition->getSetting('field');
        if (
          $fieldName !== $fieldDefinition->getName() ||
          !$definitions[$targetFieldName] instanceof ThirdPartySettingsInterface
        ) {
          // Field already targeted, or it doesn't support 3rd-party settings.
          unset($definitions[$targetFieldName]);
        }
        // Exclude 'user_selected_formatter' fields.
        unset($definitions[$fieldName]);
      }
    }

    $element['field'] = [
      '#type' => 'select',
      '#title' => $this->t('Field to be formatted'),
      '#default_value' => $this->getSetting('field'),
      '#required' => TRUE,
      '#options' => array_map(
        fn(FieldDefinitionInterface $definition): string|\Stringable => $definition->getLabel(),
        $definitions,
      ),
    ];

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public static function calculateDependencies(FieldDefinitionInterface $field_definition): array {
    $dependencies = parent::calculateDependencies($field_definition);

    if ($fieldName = $field_definition->getSetting('field')) {
      $fieldConfig = FieldConfig::loadByName($field_definition->getTargetEntityTypeId(), $field_definition->getTargetBundle(), $fieldName);
      if ($fieldConfig) {
        // By adding the field to be formatted as a dependency, we make sure
        // also this field is removed when the target field is removed.
        $config = [...$dependencies['config'] ?? [], $fieldConfig->getConfigDependencyName()];
        $dependencies['config'] = array_values(array_unique($config));
      }
    }

    return $dependencies;
  }

  /**
   * Returns the entity field manager service.
   *
   * @return \Drupal\Core\Entity\EntityFieldManagerInterface
   *   The entity field manager service.
   */
  private function getFieldManager(): EntityFieldManagerInterface {
    return \Drupal::service('entity_field.manager');
  }

}
