<?php

declare(strict_types=1);

namespace Drupal\user_selected_formatter_ui\Controller;

use Drupal\Component\Render\MarkupInterface;
use Drupal\Component\Serialization\Json;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\DependencyInjection\ClassResolverInterface;
use Drupal\Core\Entity\EntityDisplayBase;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FormatterInterface;
use Drupal\Core\Field\FormatterPluginManager;
use Drupal\Core\Form\FormState;
use Drupal\Core\Url;
use Drupal\field\FieldConfigInterface;
use Drupal\user_selected_formatter\Model\Preset;
use Drupal\user_selected_formatter_ui\Form\UserSelectedFormatterUiPresetEditForm;
use Symfony\Component\DependencyInjection\Attribute\Autowire;
use Symfony\Component\HttpKernel\Exception\HttpException;

/**
 * User selected formatter presets controller.
 */
class UserSelectedFormatterUiController extends ControllerBase {

  public function __construct(
    #[Autowire(service: 'plugin.manager.field.formatter')]
    protected readonly FormatterPluginManager $formatterManager,
    protected readonly ClassResolverInterface $classResolver,
  ) {}

  /**
   * Provides a title callback for presets list page.
   *
   * @param \Drupal\field\FieldConfigInterface $field_config
   *   The field config entity.
   */
  public function getListTitle(FieldConfigInterface $field_config): MarkupInterface {
    return $this->t('Formatter presets for @field field', [
      '@field' => $field_config->getLabel(),
    ]);
  }

  /**
   * Provides a callback for presets list page.
   *
   * @param \Drupal\field\FieldConfigInterface $field_config
   *   The field config entity.
   */
  public function getList(FieldConfigInterface $field_config): array {
    if ($field_config->getType() === 'user_selected_formatter') {
      throw new HttpException(404);
    }

    $entityTypeId = $field_config->getTargetEntityTypeId();
    $bundleEntityTypeId = $this->entityTypeManager()->getDefinition($entityTypeId)->getBundleEntityType();
    $presets = $field_config->getThirdPartySetting('user_selected_formatter', 'presets', []);

    $rows = [];
    foreach ($presets as $presetId => $presetData) {
      $preset = Preset::createFromArray($presetData);
      /** @var \Drupal\Core\Field\FormatterInterface $formatter */
      $formatter = $this->formatterManager->getInstance($preset->getFormatterOptions($field_config));
      assert($formatter instanceof FormatterInterface);
      $routeParameters = [
        $bundleEntityTypeId => $field_config->getTargetBundle(),
        'field_config' => $field_config->id(),
        'preset' => $presetId,
      ];

      $rows[] = [
        'label' => $preset->label,
        'formatter' => $formatter->getPluginDefinition()['label'] ?? $formatter->getPluginId(),
        'summary' => [
          'data' => $this->getSettingsSummary($formatter, $field_config),
        ],
        'operations' => [
          'data' => [
            '#type' => 'operations',
            '#links' => [
              'edit' => [
                'title' => $this->t('Edit'),
                'url' => Url::fromRoute("entity.field_config.$entityTypeId.user_selected_formatter_preset.edit", $routeParameters),
              ],
              'delete' => [
                'title' => $this->t('Delete'),
                'url' => Url::fromRoute("entity.field_config.$entityTypeId.user_selected_formatter_preset.delete", $routeParameters),
                'attributes' => [
                  'class' => ['button', 'button--danger', 'use-ajax'],
                  'data-dialog-type' => 'modal',
                  'data-dialog-options' => Json::encode(['width' => 880]),
                ],
                'attached' => [
                  'library' => ['core/drupal.dialog.ajax'],
                ],
              ],
            ],
          ],
        ],
      ];
    }

    return [
      '#theme' => 'table',
      '#header' => [
        'label' => $this->t('Name'),
        'formatter' => $this->t('Formatter'),
        'summary' => $this->t('Summary'),
        'operations' => $this->t('Operations'),
      ],
      '#rows' => $rows,
      '#empty' => $this->t('There are no user selected formatter presets available yet.'),
    ];
  }

  /**
   * Provides a callback for preset add/edit form.
   *
   * @param \Drupal\field\FieldConfigInterface $field_config
   *   The field config entity.
   * @param string|null $preset
   *   The preset ID.
   */
  public function getEditForm(FieldConfigInterface $field_config, ?string $preset = NULL): array {
    /** @var \Drupal\user_selected_formatter_ui\Form\UserSelectedFormatterUiPresetEditForm $formObject */
    $formObject = $this->classResolver->getInstanceFromDefinition(UserSelectedFormatterUiPresetEditForm::class);
    $formObject
      ->setEntity($field_config)
      ->setStringTranslation($this->getStringTranslation())
      ->setModuleHandler($this->moduleHandler())
      ->setEntityTypeManager($this->entityTypeManager())
      ->setOperation('default')
      ->setPresetId($preset);
    $formState = new FormState();

    return $this->formBuilder()->buildForm($formObject, $formState);
  }

  /**
   * Alters the widget or formatter settings summary.
   *
   * @param \Drupal\Core\Field\FormatterInterface $formatter
   *   The formatter.
   * @param \Drupal\Core\Field\FieldDefinitionInterface $definition
   *   The field definition.
   *
   * @see \Drupal\field_ui\Form\EntityViewDisplayEditForm::alterSettingsSummary()
   */
  protected function getSettingsSummary(FormatterInterface $formatter, FieldDefinitionInterface $definition): array {
    $summary = $formatter->settingsSummary();

    $context = [
      'formatter' => $formatter,
      'field_definition' => $definition,
      'view_mode' => EntityDisplayBase::CUSTOM_MODE,
    ];
    $this->moduleHandler()->alter('field_formatter_settings_summary', $summary, $context);

    if ($summary) {
      return [
        '#type' => 'inline_template',
        '#template' => '<div class="field-plugin-summary">{{ summary|safe_join("<br />") }}</div>',
        '#context' => ['summary' => $summary],
      ];
    }

    return [];
  }

}
