<?php

namespace Drupal\user_selected_formatter_ui\Form;

use Drupal\Component\Render\MarkupInterface;
use Drupal\Component\Utility\Html;
use Drupal\Core\Ajax\AjaxFormHelperTrait;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\ReplaceCommand;
use Drupal\Core\Entity\EntityDisplayBase;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Extension\ModuleExtensionList;
use Drupal\Core\Field\FieldTypePluginManagerInterface;
use Drupal\Core\Field\FormatterPluginManager;
use Drupal\Core\Form\FormStateInterface;
use Drupal\field\FieldConfigInterface;
use Drupal\field\FieldLabelOptionsTrait;
use Drupal\user_selected_formatter\Model\Preset;
use Drupal\user_selected_formatter_ui\UserSelectionFormatterUiTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Preset add/edit form.
 */
class UserSelectedFormatterUiPresetEditForm extends EntityForm {

  use AjaxFormHelperTrait;
  use FieldLabelOptionsTrait;
  use UserSelectionFormatterUiTrait;

  /**
   * Preset being edited.
   */
  protected ?string $presetId;

  protected function __construct(
    protected FormatterPluginManager $formatterManager,
    protected EntityFieldManagerInterface $fieldManager,
    protected FieldTypePluginManagerInterface $fieldTypeManager,
    protected ModuleExtensionList $moduleList,
  ) {}

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): self {
    return new static(
      $container->get('plugin.manager.field.formatter'),
      $container->get('entity_field.manager'),
      $container->get('plugin.manager.field.field_type'),
      $container->get('extension.list.module'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state): array {
    $form = parent::form($form, $form_state);

    $form['#title'] = $this->getTitle();
    $form_state->set('user_selected_formatter_preset_is_new', !$this->presetId);

    if ($this->isAjax()) {
      // @see https://www.drupal.org/node/2897377
      $form['#id'] = Html::getId($form_state->getBuildInfo()['form_id']);
    }
    $defaults = $this->getDefaults();

    $form['#entity_builders'][] = '::setFieldConfigThirdPartySettings';
    $form['user_selected_formatter_preset'] = ['#tree' => TRUE];
    $preset =& $form['user_selected_formatter_preset'];

    $preset['label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Label'),
      '#default_value' => $defaults['label'],
      '#required' => TRUE,
      '#maxlength' => 255,
    ];

    $preset['id'] = [
      '#type' => 'machine_name',
      '#title' => $this->t('ID'),
      '#default_value' => $defaults['id'],
      // @see \Drupal\Core\Config\Entity\ConfigEntityStorage::MAX_ID_LENGTH
      '#maxlength' => 166,
      '#disabled' => $this->presetId,
      '#machine_name' => [
        'exists' => [$this, 'presetExists'],
        'source' => ['user_selected_formatter_preset', 'label'],
      ],
    ];

    $preset['formatter']['label'] = [
      '#type' => 'select',
      '#title' => $this->t('Label display for @title', [
        '@title' => $this->getEntity()->getLabel(),
      ]),
      '#options' => $this->getFieldLabelOptions(),
      '#default_value' => $defaults['formatter']['label'],
    ];

    $userInput = $form_state->getUserInput()['user_selected_formatter_preset'] ?? [];
    $formatterPluginId = $defaults['formatter']['type'] ?? $userInput['formatter']['type'] ?? NULL;

    $preset['formatter']['type'] = [
      '#type' => 'select',
      '#title' => $this->t('Formatter'),
      '#default_value' => $formatterPluginId,
      '#options' => $this->getFormatterOptions(),
      '#required' => TRUE,
      '#ajax' => [
        'callback' => '::ajaxSubmit',
      ],
    ];

    if ($formatterPluginId) {
      $this->setPluginSettingsForm($formatterPluginId, $preset, $form_state);
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  protected function actions(array $form, FormStateInterface $form_state): array {
    $actions = parent::actions($form, $form_state);
    $actions['cancel'] = [
      '#type' => 'link',
      '#title' => $this->t('Cancel'),
      '#url' => $this->getListUrl($this->getEntity()),
      '#attributes' => [
        'class' => ['button'],
      ],
    ];
    return $actions;
  }

  /**
   * {@inheritdoc}
   */
  protected function successfulAjaxSubmit(array $form, FormStateInterface $form_state): AjaxResponse {
    $selector = '[data-drupal-selector="' . $form['#attributes']['data-drupal-selector'] . '"]';
    return (new AjaxResponse())->addCommand(new ReplaceCommand($selector, $form));
  }

  /**
   * Provides an entity form entity builder to update the preset into field.
   *
   * @param string $entityTypeId
   *   The entity type ID.
   * @param \Drupal\field\FieldConfigInterface $fieldConfig
   *   The field config entity.
   * @param array $form
   *   The form render array.
   * @param \Drupal\Core\Form\FormStateInterface $formState
   *   The form state object.
   */
  public function setFieldConfigThirdPartySettings(string $entityTypeId, FieldConfigInterface $fieldConfig, array $form, FormStateInterface $formState): void {
    $trigger = $formState->getTriggeringElement();
    if ($trigger['#name'] === 'op' && $trigger['#type'] === 'submit' && $trigger['#button_type'] === 'primary') {
      $presets = $this->getPresets();
      $preset = Preset::createFromArray($formState->getValue('user_selected_formatter_preset'));
      $presets[$preset->id] = $preset->toArray();
      $fieldConfig->setThirdPartySetting('user_selected_formatter', 'presets', $presets);

      // Persist also the preset in the form object static cache.
      $this->setPresetId($preset->id);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    parent::submitForm($form, $form_state);

    $arguments = [
      '%preset' => $this->getPreset()['label'],
      '%field' => $this->getEntity()->getLabel(),
    ];
    if ($form_state->get('user_selected_formatter_preset_is_new')) {
      $this->messenger()->addStatus($this->t('Preset %preset has been added to field %field', $arguments));
    }
    else {
      $this->messenger()->addStatus($this->t('Preset %preset from field %field has been updated', $arguments));
    }

    $form_state->setRedirectUrl($this->getListUrl($this->getEntity()));
  }

  /**
   * Returns a list of formatters.
   *
   * @return array
   *   Formatters labels keyed by formatter ID.
   */
  protected function getFormatterOptions(): array {
    $options = $this->formatterManager->getOptions($this->getEntity()->getType());
    unset($options['user_selected_formatter_default']);
    return $options;
  }

  /**
   * Sets the formatter plugin settings form elements.
   *
   * @param string $formatterPluginId
   *   The formatter plugin ID.
   * @param array $form
   *   The complete form which will be altered.
   * @param \Drupal\Core\Form\FormStateInterface $formState
   *   The form state object.
   */
  protected function setPluginSettingsForm(string $formatterPluginId, array &$form, FormStateInterface $formState): void {
    $definition = $this->getEntity();
    $defaults = $this->getDefaults();

    // @todo UX: Remember formatter settings if user switches formatters without
    //   saving. Maybe we can store per-formatter settings into $formState.
    if (!empty($defaults['formatter']['type']) && $formatterPluginId !== $defaults['formatter']['type']) {
      // Formatter has changed. Reinitialize its defaults.
      $defaults['formatter']['settings'] = [];
      $defaults['formatter']['third_party_settings'] = [];
    }

    $options = [
      'field_definition' => $definition,
      'configuration' => [
        'type' => $formatterPluginId,
        'label' => $defaults['formatter']['label'],
        'settings' => $defaults['formatter']['settings'],
        'third_party_settings' => $defaults['formatter']['third_party_settings'],
        'weight' => 0,
      ],
      'view_mode' => EntityDisplayBase::CUSTOM_MODE,
    ];

    /** @var \Drupal\Core\Field\FormatterInterface $formatter */
    $formatter = $this->formatterManager->getInstance($options);

    $form['formatter']['settings'] = [];
    if ($settingsForm = $formatter->settingsForm($form, $formState)) {
      $form['formatter']['settings'] = [
        '#type' => 'details',
        '#open' => TRUE,
        '#title' => $this->t('@formatter settings', [
          '@formatter' => $formatter->getPluginDefinition()['label'],
        ]),
      ] + $settingsForm;
    }

    $thirdPartySettingsForm = [];
    $this->moduleHandler->invokeAllWith(
      'field_formatter_third_party_settings_form',
      function (callable $hook, string $module) use (&$thirdPartySettingsForm, $formatter, $definition, &$form, $formState) {
        $thirdPartySettingsForm[$module] = $hook(
          $formatter,
          $definition,
          EntityDisplayBase::CUSTOM_MODE,
          $form,
          $formState,
        );
      }
    );

    // Remove empty module settings.
    $thirdPartySettingsForm = array_filter($thirdPartySettingsForm);

    if ($thirdPartySettingsForm) {
      $form['formatter']['third_party_settings'] = [
        '#type' => 'container',
      ];
      foreach ($thirdPartySettingsForm as $module => $moduleSettings) {
        $form['formatter']['third_party_settings'][$module] = [
          '#type' => 'details',
          '#open' => TRUE,
          '#title' => $this->t('Settings provided by @module module', [
            '@module' => $this->moduleList->getName($module),
          ]),
        ] + $moduleSettings;
      }
    }
  }

  /**
   * Provides the page/form title.
   *
   * @return \Drupal\Component\Render\MarkupInterface
   *   The page/form title.
   */
  protected function getTitle(): MarkupInterface {
    if ($preset = $this->getPreset()) {
      return $this->t('Edit %preset preset of %field field', [
        '%preset' => $preset['label'],
        '%field' => $this->getEntity()->getLabel(),
      ]);
    }
    return $this->t('Add a new preset for %field field', [
      '%field' => $this->getEntity()->getLabel(),
    ]);
  }

  /**
   * Returns the default values for the preset if any.
   *
   * @return array
   *   Default values.
   */
  protected function getDefaults(): array {
    if ($preset = $this->getPreset()) {
      return Preset::createFromArray($preset)->toArray();
    }
    return Preset::getDefaults();
  }

  /**
   * Sets the preset ID.
   *
   * @param string|null $presetId
   *   The preset ID.
   *
   * @return $this
   */
  public function setPresetId(?string $presetId): self {
    $this->presetId = $presetId;
    return $this;
  }

  /**
   * Checks whether a given preset already exists.
   *
   * @param string $presetId
   *   The preset ID.
   *
   * @return bool
   *   Whether the preset with the given ID already exists.
   */
  public function presetExists(string $presetId): bool {
    return isset($this->getPresets()[$presetId]);
  }

  /**
   * Returns the current preset.
   *
   * @return array|null
   *   Preset data or NULL if no preset is set.
   */
  protected function getPreset(): ?array {
    return $this->presetId ? $this->getPresets()[$this->presetId] : NULL;
  }

  /**
   * Returns all presets of the current field.
   *
   * @return array
   *   A list of preset data arrays keyed by preset ID.
   */
  protected function getPresets(): array {
    return $this->getEntity()->getThirdPartySetting('user_selected_formatter', 'presets', []);
  }

  /**
   * {@inheritdoc}
   */
  public function getEntity(): FieldConfigInterface {
    // Override parent only to get nice strict typing.
    $entity = parent::getEntity();
    assert($entity instanceof FieldConfigInterface);
    return $entity;
  }

}
