<?php

declare(strict_types=1);

namespace Drupal\user_selected_formatter_ui\Routing;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Routing\RouteSubscriberBase;
use Drupal\Core\Routing\RoutingEvents;
use Drupal\user_selected_formatter_ui\Controller\UserSelectedFormatterUiController;
use Drupal\user_selected_formatter_ui\Form\UserSelectedFormatterUiPresetDeleteForm;
use Symfony\Component\Routing\RouteCollection;

/**
 * Adds the user selected formatter presets route.
 */
class UserSelectedFormatterUiRouteSubscriber extends RouteSubscriberBase {

  public function __construct(
    protected ModuleHandlerInterface $moduleHandler,
    protected EntityTypeManagerInterface $entityTypeManager,
  ) {}

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    return [
      // Act after Field UI route subscriber.
      // @see \Drupal\field_ui\Routing\RouteSubscriber::alterRoutes()
      RoutingEvents::ALTER => ['onAlterRoutes', -110],
    ];
  }

  /**
   * {@inheritdoc}
   */
  protected function alterRoutes(RouteCollection $collection): void {
    foreach ($this->entityTypeManager->getDefinitions() as $entityTypeId => $entityType) {
      if (!($name = $entityType->get('field_ui_base_route')) || !$collection->get($name)) {
        continue;
      }

      if ($this->addPresetListRoute($collection, $entityTypeId)) {
        $this->addPresetAddRoute($collection, $entityTypeId);
        $this->addPresetEditRoute($collection, $entityTypeId);
        $this->addPresetDeleteRoute($collection, $entityTypeId);
      }
    }
  }

  /**
   * Adds the preset list route.
   *
   * @param \Symfony\Component\Routing\RouteCollection $collection
   *   Route collection.
   * @param string $entityTypeId
   *   The entity type ID.
   *
   * @return bool
   *   Whether the entity type supports presets.
   */
  protected function addPresetListRoute(RouteCollection $collection, string $entityTypeId): bool {
    return $this->addPresetRouteHelper($collection, $entityTypeId, '', '', [
      '_controller' => UserSelectedFormatterUiController::class . '::getList',
      '_title_callback' => UserSelectedFormatterUiController::class . '::getListTitle',
    ]);
  }

  /**
   * Adds the preset add route.
   *
   * @param \Symfony\Component\Routing\RouteCollection $collection
   *   Route collection.
   * @param string $entityTypeId
   *   The entity type ID.
   */
  protected function addPresetAddRoute(RouteCollection $collection, string $entityTypeId): void {
    $this->addPresetRouteHelper($collection, $entityTypeId, 'add', 'add', [
      '_controller' => UserSelectedFormatterUiController::class . '::getEditForm',
    ]);
  }

  /**
   * Adds the preset edit route.
   *
   * @param \Symfony\Component\Routing\RouteCollection $collection
   *   Route collection.
   * @param string $entityTypeId
   *   The entity type ID.
   */
  protected function addPresetEditRoute(RouteCollection $collection, string $entityTypeId): void {
    $this->addPresetRouteHelper(
      $collection,
      $entityTypeId,
      'edit/{preset}',
      'edit',
      [
        '_controller' => UserSelectedFormatterUiController::class . '::getEditForm',
      ],
      [
        'preset' => '^[a-z0-9_]+$',
      ],
    );
  }

  /**
   * Adds the preset delete route.
   *
   * @param \Symfony\Component\Routing\RouteCollection $collection
   *   Route collection.
   * @param string $entityTypeId
   *   The entity type ID.
   */
  protected function addPresetDeleteRoute(RouteCollection $collection, string $entityTypeId): void {
    $this->addPresetRouteHelper(
      $collection,
      $entityTypeId,
      'delete/{preset}',
      'delete',
      [
        '_form' => UserSelectedFormatterUiPresetDeleteForm::class,
      ],
      [
        'preset' => '^[a-z0-9_]+$',
      ],
    );
  }

  /**
   * Provides a helper to build the user selected formatter presets routes.
   *
   * @param \Symfony\Component\Routing\RouteCollection $collection
   *   The route collection.
   * @param string $entityTypeId
   *   The entity type ID.
   * @param string $routePathSuffix
   *   The suffix to be appended to the route name.
   * @param string $routeNameSuffix
   *   The suffix to be appended to the route path.
   * @param array $defaults
   *   Additional defaults.
   * @param array $requirements
   *   Additional requirements.
   *
   * @return bool
   *   Whether the entity type supports presets.
   */
  protected function addPresetRouteHelper(
    RouteCollection $collection,
    string $entityTypeId,
    string $routePathSuffix,
    string $routeNameSuffix,
    array $defaults = [],
    array $requirements = [],
  ): bool {
    $fieldConfigRoute = $collection->get("entity.field_config.{$entityTypeId}_field_edit_form");
    if (!$fieldConfigRoute || !$fieldConfigRoute->hasOption('_field_ui')) {
      return FALSE;
    }

    $routePathSuffix = $routePathSuffix ? "/$routePathSuffix" : '';
    $routeNameSuffix = $routeNameSuffix ? ".$routeNameSuffix" : '';
    $routeName = "entity.field_config.$entityTypeId.user_selected_formatter_preset$routeNameSuffix";
    $route = clone $fieldConfigRoute;
    $route->setPath($fieldConfigRoute->getPath() . "/format-presets$routePathSuffix");

    $fieldConfigRouteDefaults = $fieldConfigRoute->getDefaults();
    unset($fieldConfigRouteDefaults['_entity_form']);

    $defaults += $fieldConfigRouteDefaults;
    $route->setDefaults($defaults);

    $route->setRequirements($requirements + $fieldConfigRoute->getRequirements());

    $options = $fieldConfigRoute->getOptions();
    $options['parameters']['field_config']['type'] = 'entity:field_config';
    if (str_contains($route->getPath(), '{preset}')) {
      $options['parameters']['preset']['type'] = 'user_selected_formatter_preset';
    }
    $route->setOptions($options);

    $collection->add($routeName, $route);

    return TRUE;
  }

}
